from tensorflow.keras.utils import Sequence
import numpy as np
from PIL import Image, ImageOps, ImageFile
ImageFile.LOAD_TRUNCATED_IMAGES = True



def crop_generator(images,boxes,resize=299,buffer=0,batch_size=32,standardize=True):
    """
    Yields the next training batch.
    Suppose `samples` is an array [[image1_filename,label1], [image2_filename,label2],...].
    """
    num_samples = len(images)
    batch_size=int(batch_size)
    resize=int(resize)
    #offset=0
    while True: # Loop forever so the generator never terminates
        #shuffle(samples)
        # Get index to start each batch: [0, batch_size, 2*batch_size, ..., max multiple of batch_size <= num_samples]
        for offset in range(0, num_samples, batch_size):
            # Initialise X_train and y_train arrays for this batch
            X_eval = []

            # For each image
            for i in range(offset,min(num_samples,offset+batch_size)):
                # Load image (X) and label (y)
                try: img=Image.open(images[i])
                except OSError: continue
                width, height = img.size

                left = width*boxes[i,0]
                top = height*boxes[i,1]
                right = width*(boxes[i,0]+boxes[i,2])
                bottom = height*(boxes[i,1]+boxes[i,3])

                buffer2=max(right-left,bottom-top)*buffer

                left = max(0,left-buffer2)
                top = max(0,top-buffer2)
                right = min(width,right+buffer2)
                bottom = min(height,bottom+buffer2)

                img=img.crop((left, top, right, bottom))
                img=resize_with_padding(img, (resize,resize))
                #img=img.resize((resize,resize))
                    
                # apply any kind of preprocessing                img = cv2.resize(img,(resize,resize))
                # Add example to arrays
                if standardize:
                    X_eval.append(np.array(img)/255)
                else:
                    X_eval.append(np.array(img))

            # Make sure they're numpy arrays (as opposed to lists)
            X_eval = np.array(X_eval)

            # The generator-y part: yield the next training batch            
            yield(X_eval)
            #offset+=batch_size



def resize_with_padding(img, expected_size):
    #img.thumbnail((expected_size[0], expected_size[1]))
    if img.size[0] == 0 or img.size[1] == 0: return img
    if img.size[0]>img.size[1]:
        new_size=(expected_size[0],int(expected_size[1]*img.size[1]/img.size[0]))
    else:
        new_size=(int(expected_size[0]*img.size[0]/img.size[1]),expected_size[1])
    img = img.resize(new_size, Image.BILINEAR) #NEAREST BILINEAR
    # print(img.size)
    delta_width = expected_size[0] - img.size[0]
    delta_height = expected_size[1] - img.size[1]
    pad_width = delta_width // 2
    pad_height = delta_height // 2
    padding = (pad_width, pad_height, delta_width - pad_width, delta_height - pad_height)
    return ImageOps.expand(img, padding)


class GenerateCropsFromFile(Sequence):
    def __init__(self, x, boxes, resize=299,buffer=2,batch_size=32,standardize=True):
        self.x = x
        self.boxes = boxes
        self.resize = int(resize)
        self.buffer=buffer
        self.batch_size = int(batch_size)
        self.standardize=standardize
        

    def __len__(self):
        return int(np.ceil(len(self.x) / float(self.batch_size)))

    def __getitem__(self, idx):
        imgarray = []
        for i in range(min(len(self.x),idx * self.batch_size),min(len(self.x),(idx + 1) * self.batch_size)):
        #for i in range(idx * self.batch_size,(idx + 1) *self.batch_size):
        #for i in range(1,2):
            try: img=Image.open(self.x[i])
            except OSError: continue
            width, height = img.size
            
            left = width*self.boxes[i,0]
            top = height*self.boxes[i,1]
            right = width*(self.boxes[i,0]+self.boxes[i,2])
            bottom = height*(self.boxes[i,1]+self.boxes[i,3])
        #    print(left,right,top,bottom)

            buffer2=max(right-left,bottom-top)*self.buffer

            left = max(0,left-buffer2)
            top = max(0,top-buffer2)
            right = min(width,right+buffer2)
            bottom = min(height,bottom+buffer2)
            
            img=img.crop((left, top, right, bottom))
            img=resize_with_padding(img, (self.resize,self.resize))
            if self.standardize:
                imgarray.append(np.array(img)/255)
            else:
                imgarray.append(np.array(img))
            #img=np.array(img)
            #imgarray.append(np.expand_dims(img, axis=0)/255)
        
        return np.array(imgarray)




"""
Module to run a TensorFlow animal detection model on images.

The class TFDetector contains functions to load a TensorFlow detection model and run inference.
The main function in this script also render the predicted bounding boxes on images and
save the resulting images (with bounding boxes).

This script is not a good way to process lots of images (tens of thousands, say).
It does not facilitate checkpointing the results so if it crashes
you would have to start from scratch. If you want to run a detector (e.g. ours)
on lots of images, you should check out:

1) run_tf_detector_batch.py (for local execution)

2) https://github.com/microsoft/CameraTraps/tree/master/api/batch_processing
   (for running large jobs on Azure ML)

To run this script, we recommend you set up a conda virtual environment following instructions
in the Installation section on the main README, using `environment-detector.yml` as the
environment file where asked.

This is a good way to test our detector on a handful of images and
get super-satisfying, graphical results.  It's also a good way to see how
fast a detector model will run on a particular machine.

If you would like to *not* use the GPU on the machine, set the environment variable CUDA_VISIBLE_DEVICES to "-1"

If no output directory is specified, writes detections for c:\foo\bar.jpg to
c:\foo\bar_detections.jpg .

This script will only consider detections with > 0.1 confidence at all times. The `threshold` you
provide is only for rendering the results. If you need to see lower-confidence detections, you can change
DEFAULT_OUTPUT_CONFIDENCE_THRESHOLD.

Reference:
https://github.com/tensorflow/models/blob/master/research/object_detection/inference/detection_inference.py
"""

#%% Constants, imports, environment
import os
import warnings
from datetime import datetime
import humanfriendly
import numpy as np
from tqdm import tqdm


# ignoring all "PIL cannot read EXIF metainfo for the images" warnings
warnings.filterwarnings('ignore', '(Possibly )?corrupt EXIF data', UserWarning)

# Metadata Warning, tag 256 had too many entries: 42, expected 1
warnings.filterwarnings('ignore', 'Metadata warning', UserWarning)

# Numpy FutureWarnings from tensorflow import
warnings.filterwarnings('ignore', category=FutureWarning)

os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2' 

import tensorflow as tf
tf.get_logger().setLevel('ERROR')
tf.autograph.set_verbosity(3)


def truncate_float(x, precision=3):
    assert precision > 0
    if np.isclose(x, 0):
        return 0
    else:
        factor = math.pow(10, precision - 1 - math.floor(math.log10(abs(x))))
        return math.floor(x * factor)/factor


class TFDetector:
    """
    A detector model loaded at the time of initialization. It is intended to be used with
    the MegaDetector (TF). The inference batch size is set to 1; code needs to be modified
    to support larger batch sizes, including resizing appropriately.
    """

    # Number of decimal places to round to for confidence and bbox coordinates
    CONF_DIGITS = 3
    COORD_DIGITS = 4

    # MegaDetector was trained with batch size of 1, and the resizing function is a part
    # of the inference graph
    BATCH_SIZE = 1

    # An enumeration of failure reasons
    FAILURE_TF_INFER = 'Failure TF inference'
    FAILURE_IMAGE_OPEN = 'Failure image access'

    DEFAULT_RENDERING_CONFIDENCE_THRESHOLD = 0.85  # to render bounding boxes
    DEFAULT_OUTPUT_CONFIDENCE_THRESHOLD = 0.1  # to include in the output json file

    DEFAULT_DETECTOR_LABEL_MAP = {
        '1': 'animal',
        '2': 'person',
        '3': 'vehicle'  # will be available in megadetector v4
    }

    NUM_DETECTOR_CATEGORIES = 4  # animal, person, group, vehicle - for color assignment

    def __init__(self, model_path):
        """Loads the model at model_path and start a tf.Session with this graph. The necessary
        input and output tensor handles are obtained also."""
        detection_graph = TFDetector.__load_model(model_path)
        self.tf_session = tf.compat.v1.Session(graph=detection_graph)

        self.image_tensor = detection_graph.get_tensor_by_name('image_tensor:0')
        self.box_tensor = detection_graph.get_tensor_by_name('detection_boxes:0')
        self.score_tensor = detection_graph.get_tensor_by_name('detection_scores:0')
        self.class_tensor = detection_graph.get_tensor_by_name('detection_classes:0')

    @staticmethod
    def round_and_make_float(d, precision=4):
        return truncate_float(float(d), precision=precision)

    @staticmethod
    def __convert_coords(np_array):
        """ Two effects: convert the numpy floats to Python floats, and also change the coordinates from
        [y1, x1, y2, x2] to [x1, y1, width_box, height_box] (in relative coordinates still).

        Args:
            np_array: array of predicted bounding box coordinates from the TF detector

        Returns: array of predicted bounding box coordinates as Python floats and in [x1, y1, width_box, height_box]

        """
        # change from [y1, x1, y2, x2] to [x1, y1, width_box, height_box]
        width_box = np_array[3] - np_array[1]
        height_box = np_array[2] - np_array[0]

        new = [np_array[1], np_array[0], width_box, height_box]  # cannot be a numpy array; needs to be a list

        # convert numpy floats to Python floats
        for i, d in enumerate(new):
            new[i] = TFDetector.round_and_make_float(d, precision=TFDetector.COORD_DIGITS)
        return new

    @staticmethod
    def __load_model(model_path):
        """Loads a detection model (i.e., create a graph) from a .pb file.

        Args:
            model_path: .pb file of the model.

        Returns: the loaded graph.

        """
        print('TFDetector: Loading graph...')
        detection_graph = tf.Graph()
        with detection_graph.as_default():
            od_graph_def = tf.compat.v1.GraphDef()
            with tf.io.gfile.GFile(model_path, 'rb') as fid:
                serialized_graph = fid.read()
                od_graph_def.ParseFromString(serialized_graph)
                tf.import_graph_def(od_graph_def, name='')
        print('TFDetector: Detection graph loaded.')

        return detection_graph

    def _generate_detections_one_image(self, image):
        np_im = np.asarray(image, np.uint8)
        im_w_batch_dim = np.expand_dims(np_im, axis=0)

        # need to change the above line to the following if supporting a batch size > 1 and resizing to the same size
        # np_images = [np.asarray(image, np.uint8) for image in images]
        # images_stacked = np.stack(np_images, axis=0) if len(images) > 1 else np.expand_dims(np_images[0], axis=0)

        # performs inference
        (box_tensor_out, score_tensor_out, class_tensor_out) = self.tf_session.run(
            [self.box_tensor, self.score_tensor, self.class_tensor],
            feed_dict={self.image_tensor: im_w_batch_dim})

        return box_tensor_out, score_tensor_out, class_tensor_out

    def generate_detections_one_image(self, image, image_id,
                                      detection_threshold=DEFAULT_OUTPUT_CONFIDENCE_THRESHOLD):
        """Apply the detector to an image.

        Args:
            image: the PIL Image object
            image_id: a path to identify the image; will be in the `file` field of the output object
            detection_threshold: confidence above which to include the detection proposal

        Returns:
        A dict with the following fields, see https://github.com/microsoft/CameraTraps/tree/siyu/inference_refactor/api/batch_processing#batch-processing-api-output-format
            - image_id (always present)
            - max_detection_conf
            - detections, which is a list of detection objects containing `category`, `conf` and `bbox`
            - failure
        """
        result = {
            'file': image_id
        }
        try:
            b_box, b_score, b_class = self._generate_detections_one_image(image)

            # our batch size is 1; need to loop the batch dim if supporting batch size > 1
            boxes, scores, classes = b_box[0], b_score[0], b_class[0]

            detections_cur_image = []  # will be empty for an image with no confident detections
            max_detection_conf = 0.0
            for b, s, c in zip(boxes, scores, classes):
                if s > detection_threshold:
                    detection_entry = {
                        'category': str(int(c)),  # use string type for the numerical class label, not int
                        'conf': truncate_float(float(s),  # cast to float for json serialization
                                               precision=TFDetector.CONF_DIGITS),
                        'bbox': TFDetector.__convert_coords(b)
                    }
                    detections_cur_image.append(detection_entry)
                    if s > max_detection_conf:
                        max_detection_conf = s

            result['max_detection_conf'] = truncate_float(float(max_detection_conf),
                                                          precision=TFDetector.CONF_DIGITS)
            result['detections'] = detections_cur_image
            
            result['date'] = datetime.fromtimestamp(os.path.getmtime(image_id)).strftime('%Y-%m-%d')
        except Exception as e:
            result['failure'] = TFDetector.FAILURE_TF_INFER
            print('TFDetector: image {} failed during inference: {}'.format(image_id, str(e)))

        return result



# modified from CameraTraps/MegaDetector
import TFDetector
import tqdm
import cv2
import json
from FileManagement import load_image


def load_and_run_detector_batch(image_file_names, checkpoint_path,
                                confidence_threshold, checkpoint_frequency, results, detector_file):

    already_processed = set([i['file'] for i in results])

    # load the detector
    tf_detector = TFDetector(detector_file)

    count = 0  # does not count those already processed
    for im_file in tqdm(image_file_names):
        if im_file in already_processed: continue # will not add additional entries not in the starter checkpoint
        if im_file.lower().endswith(".mp4"):
            try:
                _,image = cv2.VideoCapture(im_file).read()  #get first frame
            except Exception as e:
                print('Video {} cannot be loaded. Exception: {}'.format(im_file, e))
                result = {
                    'file': im_file,
                    'failure': TFDetector.FAILURE_IMAGE_OPEN
                }
                results.append(result)
                continue
        else: 
            try:
                image = load_image(im_file)
            except Exception as e:
                print('Image {} cannot be loaded. Exception: {}'.format(im_file, e))
                result = {
                    'file': im_file,
                    'failure': TFDetector.FAILURE_IMAGE_OPEN
                }
                results.append(result)
                continue

        try:
            result = tf_detector.generate_detections_one_image(image, im_file, detection_threshold=confidence_threshold)
            results.append(result)
            count += 1

        except Exception as e:
            print('An error occurred while running the detector on image {}. Exception: {}'.format(im_file, e))
            continue

        # checkpoint
        if checkpoint_frequency != -1 and count % checkpoint_frequency == 0:
            print('Writing a new checkpoint after having processed {} images since last restart'.format(count))
            with open(checkpoint_path, 'w') as f:
                json.dump({'images': results}, f)

    return results  # actually modified in place





# -*- coding: utf-8 -*-
"""
Rename and resize images, create a subject set and upload new subjects to it
Created on Mon Jul 11 15:34:53 2016

@author: Mathias Tobler

Copyright (C) 2017 Mathias Tobler

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Based on code by:
________________________________________________
Stijn Calders
Space Physics - Space Weather

Royal Belgian Institute for Space Aeronomy (BIRA-IASB)
Ringlaan 3
B-1180 Brussels
BELGIUM

phone  : +32 (0)2 373.04.19
e-mail : stijn.calders@aeronomie.be
web    : www.aeronomie.be
________________________________________________
"""

from panoptes_client import SubjectSet, Subject, Project, Panoptes
import glob
import os
import time
import sys
from datetime import datetime
import pandas as pd
from PIL import Image
from PIL import ImageFile
ImageFile.LOAD_TRUNCATED_IMAGES = True
import argparse

def copy_image(infile,outpath,imageheight):
    img = Image.open(infile.FilePath)
    try: exif = img.info['exif']
    except KeyError: return None
 
    wpercent = (imageheight / float(img.size[1]))
    imagewidth = int((float(img.size[0]) * float(wpercent)))
    img = img.resize((imagewidth, imageheight), Image.ANTIALIAS)
    outfile = outpath+infile.FileName
    img.save(outfile,"JPEG",exif=exif)
    return outfile

def create_SubjectSet(project,subject_set_name):
    subject_set = SubjectSet()
    subject_set.links.project = project
    subject_set.display_name = subject_set_name 
    subject_set.save()
    return subject_set

def connect_to_Panoptes(usr,pw):
    Panoptes.connect(username=usr, password=pw)
    print("Connected to Zooniverse")

def upload_to_Zooniverse_Simple(project_name,subject_set_name,images,outdir,imageheight=700): 
    sys.stdout.flush()
    #Create a new subject set or append the subjects to an existing one
    project = Project.find(project_name)
    print("Connected to ", project)
    subject_set = SubjectSet.find(int(subject_set_name))

    print(subject_set)
    print("Uploading images...")
    print(images)
    for _,infile in images.iterrows():
        if not os.path.exists(infile.FileName): 
            print(infile.FileName)
        subject = Subject() #create new subject
        subject.links.project = project
        outfile=copy_image(infile,outdir,imageheight)
        if outfile == None:
            continue
        subject.add_location(outfile)
        subject.metadata['OrigName'] = infile.FileName
        subject.metadata['!OrigFolder'] = infile.FilePath
        subject.metadata['DateTime'] = str(infile.DateTime)
        subject.metadata['#machine_prediction'] = infile.ZooniverseCode #map to zooniverse name
        subject.metadata['#machine_confidence'] = infile.confidence
        subject.save()
        subject_set.add(subject)



def upload_to_Zooniverse(project_name,subject_set_name,images,outdir,maxSeq=1,maxTime=0,imageheight=700): 
    sys.stdout.flush()
    #Create a new subject set or append the subjects to an existing one
    project = Project.find(project_name)
    print("Connected to ", project)
    subject_set = SubjectSet.find(int(subject_set_name))

    print(subject_set)
    print("Uploading images...")
    
   # print(images.columns.values)

    si=0 #set sequence counter to 0
    for _,infile in images.iterrows():
        bProc=False
        try:
            currentDateTime = time.mktime(time.strptime(infile.DateTime,"%Y-%m-%d %H:%M:%S"))
        except TypeError:
            currentDateTime = datetime.strptime(str(infile.DateTime),"%Y-%m-%d %H:%M:%S")

        print(currentDateTime)

        # add image to current sequence 
        if si > 0 and si < maxSeq and (currentDateTime-dDateTime) <= maxTime and subject.metadata['#machine_prediction'] == infile.ZooniverseCode:       
         # print("Continue")
          outfile=copy_image(infile,outdir,imageheight)
          if outfile == None:
               continue
          subject.add_location(outfile)
          subject.metadata['OrigName'] = subject.metadata['OrigName']+";"+ infile.FileName
          subject.metadata['!OrigFolder'] = subject.metadata['!OrigFolder']+";"+ infile.FilePath
          subject.metadata['DateTime'] = subject.metadata['DateTime']+";"+ infile.DateTime.strftime("%Y-%m-%d %H:%M:%S")
          #subject.metadata['NewName'] = subject.metadata['NewName']+";"+ infile.NewName
          dDateTime=currentDateTime
          si=si+1
          bProc=True
        #  print(subject.metadata)

        #sequence has ended, current image is in a new sequence
        elif si > 0 and si < maxSeq and ((currentDateTime-dDateTime) > maxTime or subject.metadata['#machine_prediction'] != infile.ZooniverseCode):
          si=maxSeq 
          bProc=False

                #need to start a new sequence OR maxSeq = 1 
        if si==maxSeq:  
         # print("Upload")
          subject.save()
          subject_set.add(subject)
          si=0

        # first image within a sequence 
        if si==0 and bProc==False: 
          subject = Subject() #create new subject
          subject.links.project = project
          dDateTime=currentDateTime
          outfile=copy_image(infile,outdir,imageheight)
          if outfile == None:
               continue
          subject.add_location(outfile)
          # You can set whatever metadata you want, or none at all
          subject.metadata['OrigName'] = infile.FileName
          subject.metadata['!OrigFolder'] = infile.FilePath
          subject.metadata['DateTime'] = infile.DateTime.strftime("%Y-%m-%d %H:%M:%S")
          subject.metadata['#machine_prediction'] = infile.ZooniverseCode #map to zooniverse name
          subject.metadata['#machine_confidence'] = infile.md_confidence
          #subject.metadata['NewName'] = infile.NewName
          si=1



from DetectMD import load_and_run_detector_batch
from PIL import Image
import numpy as np
from ImageCropGenerator import GenerateCropsFromFile


def predict_images(images, model, tf_file, confidence=0.5):
    if len(images) > 0:
        results = load_and_run_detector_batch(images, checkpoint_path="NA", confidence_threshold=confidence,
                                              checkpoint_frequency=-1, results=[], detector_file=tf_file)
        print(results)

        # Make a list of File names from the results
        # Make a list of bounding boxes
        bboxes = []
        filenames = []
        for dictionary in results:
            detections = dictionary['detections']
            if len(detections) > 0:
                detection_dictionary = detections[0]
                bboxes.append(detection_dictionary['bbox'])

                image = Image.open(dictionary['file'])
                image_path = f"image_{detection_dictionary['bbox']}.jpg"

                image.save(image_path)
                filenames.append(image_path)

        # Make bboxes into array
        bboxes = np.array(bboxes)

        # Create Generator
        if len(filenames) > 0:
            generator = GenerateCropsFromFile(filenames, bboxes)
            predictions = model.predict(generator)
            return predictions



from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
import time

options = webdriver.ChromeOptions()
options.add_argument("--enable-javascript")
driver = webdriver.Chrome(options=options)
driver.get("https://www.strikeforcewireless.com/login?redirect=%2Fcameras")
name= driver.find_element_by_id("username")
name.clear()
name.send_keys()
pw = driver.find_element_by_id("password")
pw.send_keys()
pw.send_keys(Keys.RETURN)

#wait to login
time.sleep(2)

#navigate to photos page
driver.get("https://www.strikeforcewireless.com/photos")

#wait for javascript to load
time.sleep(2)

images = driver.find_elements_by_tag_name('img')
for image in images:
    print(image.get_attribute('src'))
driver.close()




import os
import pandas as pd
from tqdm import tqdm


def SortAndCopy(results,outdir):
  print("Copying files...")
  for image in tqdm(results):
    
    source = Path(image["FilePath"])
    base = image["FileName"]
        
    classfolder = outdir + '/' + image["species"] + '/'
    #create classfolder if it doesnt yet exist  
    if not os.path.isdir(Path(classfolder)): os.makedirs(Path(classfolder))    
       
    date = image['FileModifyDate'].strftime('%Y-%m-%d')
    
    dest = Path(classfolder + date + "_" + base)
    
    if os.path.isfile(source):     
      if os.path.isfile(dest):
        print("File already exists! ",dest)
        pass
      else: shutil.copy2(source, dest)
    else: print(f"Error: File {source} not found!")


def SortAndCopy_MD(results):
  print("Copying files...")
  for image in tqdm(results):
      
    base = os.path.basename(image["file"])
  
    detex = image.get('max_detection_conf',-1)
    
    if detex < 0: continue #error
  
    elif detex > 0 : #object detected
        best = image['detections'][0]
        dest = args.output_dir + '/' + classes[int(best["category"])] + '/'
        
    else: # object not detected 
        dest = args.output_dir + '/' + classes[0] + '/'
        
    if not os.path.isdir(Path(dest)): os.makedirs(Path(dest))    
   
    source = image["file"]
  
    source = Path(source)
  
    dest = Path(dest + image['date'] + "_" + base)
    if os.path.isfile(source):     
        if os.path.isfile(dest):
            print("File already exists! ",dest)
            pass
        else: shutil.copy2(source, dest)
    else:
        print(source)
        print("File not found!")



import pandas as pd
import os
import glob
import cv2
import random
from PIL import Image
from datetime import datetime
import multiprocessing as mp


def load_image(input_file):
    """
    from CameraTraps/visualization/visualization_utils.py
    """
    image = Image.open(input_file)
    if image.mode not in ('RGBA', 'RGB', 'L'):
        raise AttributeError('Input image {} uses unsupported mode {}'.format(input_file, image.mode))
    if image.mode == 'RGBA' or image.mode == 'L':
        # PIL.Image.convert() returns a converted copy of this image
        image = image.convert(mode='RGB')
    image.load()
    return image


def is_image(s):
    """
    from CameraTraps/MegaDetector
    Check a file's extension against a hard-coded set of image file extensions    '
    """
    image_extensions = ['.jpg', '.jpeg', '.gif', '.png', '.mp4']
    ext = os.path.splitext(s)[1].lower()
    return ext in image_extensions


def extractImages(file_path, outdir, fps=None, frames=None):
    cap = cv2.VideoCapture(file_path)
    filename = os.path.basename(file_path)
    filename, extension = os.path.splitext(filename)
    frames_saved = []

    if fps is None:
        video_fps = cap.get(cv2.CAP_PROP_FPS)
        frame_count = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
        duration = frame_count / video_fps
        frame_capture = 0
        increment = int(frame_count / frames)
        while cap.isOpened() and (len(frames_saved) < frames):
            cap.set(cv2.cv2.CAP_PROP_POS_FRAMES, frame_capture)
            ret, frame = cap.read()
            if not ret:
                break

            out_path = outdir + filename + "-" + '{:05}'.format(random.randrange(1, 10 ** 5)) + "-" + str(
                frame_capture) + '.jpg'
            cv2.imwrite(out_path, frame)
            frames_saved.append(out_path)
            frame_capture += increment
    else:
        frame_capture = 0
        while cap.isOpened():
            ret, frame = cap.read()
            cap.set(cv2.CAP_PROP_POS_MSEC, (frame_capture * 1000))
            if not ret:
                break

            out_path = outdir + filename + "-" + '{:05}'.format(random.randrange(1, 10 ** 5)) + "-" + str(
                frame_capture) + '.jpg'
            cv2.imwrite(out_path, frame)
            frames_saved.append(out_path)
            frame_capture += fps

        cap.release()
        cv2.destroyAllWindows()

    return frames_saved


def imagesFromVideos(files, outdir=None, outfile=None, format="jpg", fps=None, frames=None, parallel=False, nproc=1):
    # assert type(files) == "DataFrame", "'files' must be Data Frame."
    if not os.path.isdir(outdir): os.makedirs(outdir)

    if (fps != None) and (frames != None):
        print("If both fps and frames are defined fps will be used.")
    assert (fps != None) or (frames != None), "Either fps or frames need to be defined."
    images = []
    videos = []
    for file in files:
        filename, extension = os.path.splitext(file)
        extension = extension.lower()
        if extension == 'jpg' or extension == 'png':
            images.append(file)
        else:
            videos.append(file)

    if parallel:
        cpu_count = mp.cpu_count()
        pool = mp.Pool(cpu_count)
        videoframes = [pool.apply(extractImages, args=(video, outdir, fps, frames)) for video in videos]
        pool.close()

    else:
        videoframes = []
        for video in videos:
            videoframes.append(extractImages(video, outdir=outdir, fps=fps, frames=frames))
    return videoframes


def buildFileManifest(imagedir, outfile=None):
    if outfile: pass
    # load file manifest
    if not os.path.isdir(imagedir):
        return "The given directory does not exist."

    files = glob.glob(os.path.join(imagedir, '**', '*.*'), recursive=True)
    images = [s for s in files if is_image(s)]

    images = pd.DataFrame(images, columns=["FilePath"])
    images["FileName"] = images["FilePath"].apply(lambda x: os.path.split(x)[1])
    images["FileModifyDate"] = images["FilePath"].apply(
        lambda x: datetime.fromtimestamp(os.path.getmtime(x)).strftime('%Y-%m-%d %H:%M:%S'))

    return images
